from django.core.management.base import BaseCommand, CommandError
from logtracker.journal.models import Entry
import os
import textwrap

def bold(txt):
    bold = "\033[1m"
    end = "\033[0m"
    return bold + txt + end


class Command(BaseCommand):
    help = "Print entries"

    def add_arguments(self, parser):
        parser.add_argument("--full", action="store_true")
        parser.add_argument("--since")
        parser.add_argument("-n", "--lines", default=100)
        parser.add_argument('options', nargs="*")

    def handle(self, *args, **options):
        _, columns = os.popen("stty size", "r").read().split()
        kwargs = {k: v for (k, v) in [o.split('=') for o in options['options']]}
        for entry in Entry.objects.dump(lines=options['lines'], since=options['since'], **kwargs):
            line = "%s %s %s %s" % (
                entry.timestamp.astimezone().strftime("%b %d %X"),
                entry.host,
                entry.unit,
                entry.data.get("MESSAGE"),
            )
            priority = entry.data.get("PRIORITY")
            if not options.get("full"):
                line = textwrap.shorten(line, int(columns))
            if priority and int(priority) < 4:
                line = bold(line)
            print(line)
